Attribute VB_Name = "basTools"
Option Explicit

Public goSettings As CSettings

Private Const WM_USER = 1024
Private Const CB_GETITEMHEIGHT = &H154
         
Private Type POINTAPI
   X As Long
   Y As Long
End Type

Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Private Type DLLVERSIONINFO
    cbSize As Long
    dwMajor As Long
    dwMinor As Long
    dwBuildNumber As Long
    dwPlatformID As Long
End Type

Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function MoveWindow Lib "user32" (ByVal hwnd As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal bRepaint As Long) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long
Private Declare Function ScreenToClient Lib "user32" (ByVal hwnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function CommonControlsVer Lib "Comctl32.dll" Alias "DllGetVersion" (pdvi As DLLVERSIONINFO) As Long
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long

' Get mouse X coordinates in pixels
'
' If a window handle is passed, the result is relative to the client area
' of that window, otherwise the result is relative to the screen
Function MouseX(Optional ByVal hwnd As Long) As Long
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    If hwnd Then ScreenToClient hwnd, lpPoint
    MouseX = lpPoint.X
End Function

' Get mouse Y coordinates in pixels
'
' If a window handle is passed, the result is relative to the client area
' of that window, otherwise the result is relative to the screen
Function MouseY(Optional ByVal hwnd As Long) As Long
    Dim lpPoint As POINTAPI
    GetCursorPos lpPoint
    If hwnd Then ScreenToClient hwnd, lpPoint
    MouseY = lpPoint.Y
End Function

Private Function GetCommonControlsVer() As String
      
    Dim tDVI As DLLVERSIONINFO
    
    tDVI.cbSize = Len(tDVI)
    On Error Resume Next    'DllGetVersion is not implemented in very early (pre v4.71) versions of comctl32.dll so an error is possible
    CommonControlsVer tDVI
    On Error GoTo 0                    'if there is an error "0.0" is returned
    GetCommonControlsVer = tDVI.dwMajor & "." & tDVI.dwMinor
    
    'ref http://msdn.microsoft.com/en-us/library/bb776779(VS.85).aspx
    '4.71 Internet Explorer 4.0
    '5.81 Comctl32.dll Windows 2000 and Windows Me. See note 3. (Win 95 with IE5.5 returns 5.81)
    '5.82 Comctl32.dll Windows XP and Windows Vista. See note 4
    'Note 4: ComCtl32.dll version 6 is not redistributable. If you want your application to use ComCtl32.dll version 6,
    ' you must add an application manifest that indicates that version 6 should be used if it is available.
            
End Function

Public Sub ComboHeightSet(oComboBox As ComboBox, ByVal Nrows%, Optional onframe As Boolean = False, Optional cbWidth%)
    
    'oComboBox the ComboBox of interest
    'Nrows% the number of rows required in the dropdown
    'onframe set to True in the call only if oComboBox is on a Frame Control
    'cbWidth% the width required in twips; if you want to change it

    Dim OldScaleMode%, rowheight%
    Dim lNewHeight&

    Const CB_GETITEMHEIGHT = &H154
    Const CBM_FIRST = &H1700    'Combobox control messages
    Const CB_SETMINVISIBLE = (CBM_FIRST + 1)
        
    With oComboBox
        
        'XP SP3 typically has Comctl32.dll v6.0, Vista SP1 has Comctl32.dll v6.16 (ref 17 July 2008)
        'CB_SETMINVISIBLE  does not seem to work with v6.0 as advertised
        If Val(GetCommonControlsVer) >= 6.16 Then
             'typically Vista and there is a manifest file specifying Comctl32.dll v6.0, default number of rows is 30
            ' note: if a manifest has been used but IsThemeActive() = 0 (Themes are disabled) the combo looks like
            '  an old style one but we still need to talk to it like this ...
            SendMessage .hwnd, CB_SETMINVISIBLE, Nrows, 0    '
        Else
            'XP and previous, or Vista with no manifest file, default number of rows is 8
            ' Change the ScaleMode on the parent to Pixels.
            OldScaleMode = .Parent.ScaleMode
            .Parent.ScaleMode = vbPixels

            rowheight = SendMessage(.hwnd, CB_GETITEMHEIGHT, 0, 0)
            lNewHeight = Nrows * rowheight + .Height + 2
            
            ' Resize the combo box window.
            If Not onframe Then
                MoveWindow .hwnd, .Left, .Top, .Width, lNewHeight, True
            Else
                'frame is unaffected by .Parent.ScaleMode and remains dimensioned in Twips so..
                MoveWindow .hwnd, .Left / Screen.TwipsPerPixelX, .Top / Screen.TwipsPerPixelY, .Width / Screen.TwipsPerPixelX, lNewHeight, True
            End If
            
            ' Replace the old ScaleMode
            .Parent.ScaleMode = OldScaleMode
        End If
        
        If cbWidth > 0 Then
            Const CB_SETDROPPEDWIDTH As Long = &H160&
            SendMessage .hwnd, CB_SETDROPPEDWIDTH, cbWidth / Screen.TwipsPerPixelX, 0
        End If
        
    End With
    
End Sub

Public Function ComboSetDropDownHeight(cbo As ComboBox, nItems As Long)

   Dim pt As POINTAPI
   Dim rc As RECT
   Dim cWidth As Long
   Dim newHeight As Long
   Dim OldScaleMode As Long
   Dim itemHeight As Long
   
  'Save the current form scalemode, then switch to pixels
   OldScaleMode = cbo.Parent.ScaleMode
   cbo.Parent.ScaleMode = vbPixels
   
  'get the system height of a single
  'combo box list item
   itemHeight = SendMessage(cbo.hwnd, CB_GETITEMHEIGHT, 0, ByVal 0)
   
  'Calculate the new height of the combo box. This
  'is the number of items times the item height
  'plus two. The 'plus two' is required to allow
  'the calculations to take into account the size
  'of the edit portion of the combo as it relates
  'to item height. In other words, even if the
  'combo is only 21 px high (315 twips), if the
  'item height is 13 px per item (as it is with
  'small fonts), we need to use two items to
  'achieve this height.
   newHeight = itemHeight * (nItems + 2)
   
   pt.X = cbo.Width
   Call ScreenToClient(cbo.Parent.hwnd, pt)
   cWidth = pt.X
   
  'get the co-ordinates of the combo box
  'relative to the screen
   Call GetWindowRect(cbo.hwnd, rc)
   pt.X = rc.Left
   pt.Y = rc.Top

  'then translate into co-ordinates
  'relative to the form.
   Call ScreenToClient(cbo.Parent.hwnd, pt)
   
  'using the values returned and set above,
  'call MoveWindow to reposition the combo box
   Call MoveWindow(cbo.hwnd, pt.X, pt.Y, cWidth, newHeight, True)
   
  'restore the original form scalemode
  'before leaving
   cbo.Parent.ScaleMode = OldScaleMode
   
End Function

'Return the file name from a path & filename
Function Filename(ByVal PathName As String) As String
    Dim Position As Long
    
    Position = InStrRev(PathName, "\", Compare:=vbTextCompare)
    
    Filename = Right$(PathName, Len(PathName) - Position)
End Function


'Return the drive\folder from a path & filename
Function FolderName(ByVal PathName As String) As String
    Dim X As Integer
    
    X = InStrRev(PathName, "\", Len(PathName) - 1, Compare:=vbTextCompare)
    FolderName = Left$(PathName, X - 1)
End Function


Sub UnloadAllForms(sFormName As String)

    Dim Form As Form
    
    For Each Form In Forms
        If Form.Name <> sFormName Then
            Unload Form
            Set Form = Nothing
        End If
    Next Form
End Sub

Function Min(vValue1 As Variant, vValue2 As Variant) As Variant
    If vValue1 < vValue2 Then
        Min = vValue1
    Else
        Min = vValue2
    End If
End Function


Function IsComboItem(cbo As ComboBox, sValue As String) As Boolean
    Dim nItem As Long
    Dim nItems As Long
    
    nItems = cbo.ListCount
    For nItem = 0 To nItems - 1
        If cbo.List(nItem) = sValue Then
            IsComboItem = True
            Exit Function
        End If
    Next
    
    IsComboItem = False
End Function

